import click
import pandas
import csv

from sidr import common


def readRunfile(runfile, taxidDict, taxDump, classificationLevel):
    unnecessaryColumns = ["Covered_bases", "Plus_reads", "Minus_reads"] # TODO: different formatting?
    contigs = []
    classList = []
    classMap = {}
    with open(runfile) as rf:
        runfile = pandas.read_csv(rf, index_col=False)
    for column in unnecessaryColumns:
        runfile.drop(column, axis=1, inplace=True)  # https://stackoverflow.com/questions/13411544/delete-column-from-pandas-dataframe for inplace
    runfile = runfile.fillna(value=False) # replace all non-existent values with False for later processing
    for row in runfile.iterrows():
        row = row[1] # iterrows returns a tuple of (index, Series)
        contigid = row["ID"]
        row.drop("ID", inplace=True)
        if not "0" == row["Origin"]:
            taxid = taxidDict[row["Origin"]] # text to taxid, should give options here
            classification = common.taxidToLineage(taxid, taxDump, classificationLevel)
            if classification not in classList:
                classList.append(classification)
        else:
            classification = False
        row.drop("Origin", inplace=True)
        variables = row.to_dict()
        contigs.append(common.Contig(contigid, variables, classification))
        for idx, className in enumerate(classList):
            classMap[className] = idx
    if len(contigs) != len(set([x.contigid for x in contigs])): # exit if duplicate contigs, https://stackoverflow.com/questions/5278122/checking-if-all-elements-in-a-list-are-unique
        raise ValueError("Input runfile contains duplicate contigIDs, exiting")
    return contigs, classMap, classList


def runAnalysis(blastdb, runfile, classificationLevel, modelOutput, output, tokeep, toremove, binary, target):
    taxDump, taxidDict = common.parseTaxdump(blastdb, True)
    contigs, classMap, classList = readRunfile(runfile, taxidDict, taxDump, classificationLevel)
    corpus, testdata, features = common.constructCorpus(contigs, classMap, binary, target)
    click.echo("Corpus constucted, %d contigs in corpus and %d contigs in test data" % (len(corpus), len(testdata)))
    classifier = common.constructModel(corpus, classList, features, modelOutput)
    result = common.classifyData(classifier, testdata, classMap)
    common.generateOutput(tokeep, toremove, result, contigs, target, output)



import click
import csv

from sklearn import tree
from sklearn import ensemble

try: # later versions of sklearn move the scoring tools
    from sklearn import model_selection as mscv
except ImportError:
    from sklearn import cross_validation as mscv


class Contig(object): # to store contigs for analysis
    def __init__(self, contigid, variables={}, classification=False):
        self.contigid = contigid
        self.variables = variables
        self.classification = classification


def parseTaxdump(blastdb, createDict):
    """
    Parses a local copy of the NCBI Taxonomy dump for use by later functions.

    Args:
        blastdb: A string containing the root directory for the NCBI taxdump.
        createDict: True to create taxon dictionary for runfile mode
    Returns:
        taxdump: A dictionary mapping NCBI taxon ids to a tuple containing the
            taxid of the parent taxon and the name of the current taxon.
        taxdict: A dictionary mapping human readable names to taxids
    """
    taxdump = {}
    taxidDict = {}
    with open(blastdb + "/names.dmp") as names:
        click.echo("Reading names.dmp")
        namesReader = list(csv.reader(names, delimiter="|"))
        with click.progressbar(namesReader) as nr: # human readable names
            for line in nr:
                if "scientific name" in line[3].strip():
                    taxdump[line[0].strip()] = line[1].strip() # {taxid: Name}
                if createDict:
                    taxidDict[line[1].strip()] = line[0].strip() # {Name: taxid}
    with open(blastdb + "/nodes.dmp") as nodes:
        click.echo("Reading nodes.dmp")
        nodesReader = list(csv.reader(nodes, delimiter="|"))
        with click.progressbar(nodesReader) as nr: # connection between taxids
            for line in nr:
                taxdump[line[0].strip()] = [taxdump[line[0].strip()], line[1].strip(), line[2].strip()]  # {taxid: [Name, Parent, Rank]}
    with open(blastdb + "/merged.dmp") as merged: # merged taxids are the same organism, effectively
        click.echo("Reading merged.dmp")
        mergedReader = list(csv.reader(merged, delimiter="|"))
        with click.progressbar(mergedReader) as mr:
            for line in mr:
                taxdump[line[0].strip()] = ["merged", line[1].strip()]
    with open(blastdb + "/delnodes.dmp") as delnodes: # if a taxid is deleted than the BLAST DB and taxdump are too far out of sync
        click.echo("Reading delnodes.dmp")
        delReader = list(csv.reader(delnodes, delimiter="|"))
        with click.progressbar(delReader) as dr:
            for line in dr:
                taxdump[line[0].strip()] = ["deleted"]
    return taxdump, taxidDict


def taxidToLineage(taxid, taxdump, classificationLevel):
    """
    Recursively converts an NCBI taxid to a full phylogenetic lineage.

    Args:
        taxid: The taxid to look up.
        taxdump: The NCBI taxonomy dump as parsed by parseTaxDump
        classificationLevel: The level of classification to save into the corpus.
    Returns:
        classification: A string containing the lineage at the chosen classification level.
    """
    taxid = taxid.split(";")[0]
    tax = taxdump[taxid]
    if "deleted" in tax: # taxid has been removed from NCBI db, indicates that taxdump and blastdb are out of sync
        raise Exception("ERROR: Taxon id %s has been deleted from the NCBI DB.\nPlease update your databases and re-run BLAST." % taxid)
    if "merged" in tax:
        merge = tax[1]
        return taxidToLineage(merge, taxdump, classificationLevel)
    try:
        if tax[2].lower() == classificationLevel.lower(): # final answer
            return tax[0]
        elif taxid != '1': # recursive call
            return taxidToLineage(tax[1], taxdump, classificationLevel)
        elif taxid == '1': # nohit
            return("nohit")
        else: # edge case
            raise Exception("Taxid %s has failed to resolve." % taxid)
    except KeyError:
        raise Exception("Taxon id %s was not found in the NCBI DB.\nPlease update your DB and try again." % taxid)


def constructCorpus(contigs, classMap, binary, target):
    """
    Construct a corpus, or body of training data for the decision tree, as well as the data under test.

    Args:
        contigs: A list of sidr.common.Contig objects with test variables.
        classMap: A dictionary mapping class names to their class id used by scikit-learn.
        binary: Set True to use "binary" (target/nontarget) classification for the model.
        target: The name of the target classification.
    Returns:
        corpus: A list of lists, containing the GC content, coverage, and class number.
        testdata: Data that was not classified by BLAST and will later be classified by the
            trained model.
        features: List of variables used by each contig.
    """
    corpus = []
    testdata = []
    features = []
    for contig in contigs:
        variableBuf = list(contig.variables.values())
        features = list(contig.variables.keys())
        if contig.classification:
            if binary:
                if contig.classification.lower() == target.lower():
                    variableBuf.append("target")
                    corpus.append(variableBuf)
                else:
                    variableBuf.append("nontarget")
                    corpus.append(variableBuf)
            else:
                variableBuf.append(contig.classification)
                corpus.append(variableBuf)
        else:
            variableBuf.insert(0, contig.contigid)
            testdata.append(variableBuf)
    return corpus, testdata, features


def constructModel(corpus, classList, features, modelOutput):
    """
    Trains a Decision Tree model on the test corpus.

    Args:
        corpus: A list of lists, containing the GC content, coverage, and class number.
        classList: A list of class names.
        features: List of variables used by each contig.
        modelOutput: Location to save model as GraphViz DOT, or False to save no model.
    Returns:
        classifier: A DecisionTreeClassifier object that has been trained on the test corpus.
    """
    corpus.sort()  # just in case
    X = []
    Y = []
    for item in corpus:
        X.append(item[:-1]) # all but the last item
        Y.append(item[-1]) # only the last item
    X_train, X_test, Y_train, Y_test = mscv.train_test_split(X, Y, test_size=0.3, random_state=0)
    # TODO: implement classifier testing and comparison, now only baggingClassifier is used as per paper
    #treeClassifier = tree.DecisionTreeClassifier()
    #treeClassifier = treeClassifier.fit(X_train, Y_train)
    #click.echo("Decision tree classifier built, score is %s out of 1.00" % treeClassifier.score(X_test, Y_test))
    baggingClassifier = ensemble.BaggingClassifier()
    baggingClassifier = baggingClassifier.fit(X_train, Y_train)
    click.echo("Bagging classifier built, score is %s out of 1.00" % baggingClassifier.score(X_test, Y_test))
    #forestClassifier = ensemble.RandomForestClassifier(n_estimators=10)
    #forestClassifier = forestClassifier.fit(X_train, Y_train)
    #click.echo("Random forest classifier built, score is %s out of 1.00" % forestClassifier.score(X_test, Y_test))
    #adaClassifier = ensemble.AdaBoostClassifier(n_estimators=100)
    #adaClassifier = adaClassifier.fit(X_train, Y_train)
    #click.echo("AdaBoost classifier built, score is %s out of 1.00" % adaClassifier.score(X_test, Y_test))
    #gradientClassifier = ensemble.GradientBoostingClassifier(n_estimators=100)
    #gradientClassifier = gradientClassifier.fit(X_train, Y_train)
    #click.echo("Gradient tree boosting classifier built, score is %s out of 1.00" % gradientClassifier.score(X_test, Y_test))
    if modelOutput:
        with open(modelOutput, 'w') as dotfile:
            tree.export_graphviz(baggingClassifier, out_file=dotfile, feature_names=features,
                                 class_names=classList, filled=True, rounded=True, special_characters=True)
    return baggingClassifier


def classifyData(classifier, testdata, classMap):
    """
    Classifies data based on the previously trained model.

    Args:
        classifier: A classifier object that has been trained on the test corpus.
        testdata: A dataset to classify based on reads.
        classMap: A dictionary mapping class names to their class id used by scikit-learn.
    Returns:
        result: Test data as classified by the model.
    """
    testdata.sort()
    X = []
    # testdata = [[contigID, variable1, variable2, ...], ...]
    for item in testdata:
        X.append(item[1::]) # all but the first item
    Y = classifier.predict(X)
    # Step one: transpose the testdata matrix and extract all contigIDs
    contigIDs = list(zip(*testdata))[0]  # https://stackoverflow.com/questions/4937491/matrix-transpose-in-python
    # Step two: combine the contigIDs with the results from the classifier
    result = list(zip(contigIDs, Y))
    return result

def generateOutput(tokeep, toremove, result, contigs, target, output):
    """
    Generates output files for completed runs.

    Args:
        tokeep: File-like object to output target contigs to.
        toremove: File-like object to output non-target contigs to.
        result: Classified data from sidr.common.classifyData().
        contigs: List of sidr.common.Contig objects from input.
        target: Target classification.
        output: File-like object to save complete results to.
    """
    targetContigIDs = []
    nontargetContigIDs = []
    outputLines = []
    for contig in contigs:
        if contig.classification:
            if target == contig.classification:
                targetContigIDs.append(contig.contigid)
            else:
                nontargetContigIDs.append(contig.contigid)
            outputLines.append([contig.contigid, contig.classification, "input"])
    for contig in result:
        if target == contig[1]:
            targetContigIDs.append(contig[0])
        else:
            nontargetContigIDs.append(contig[0])
        outputLines.append([contig[0], contig[1], "dt"])
    with open(output, "w+") as f:
        f.write("contigid, classification, source\n")
        for ln in outputLines:
            f.write("%s, %s, %s\n" % (ln[0], ln[1], ln[2]))  # https://stackoverflow.com/questions/899103/writing-a-list-to-a-file-with-python for %s\n suggestion
    if tokeep:
        with open(tokeep, "w+") as f:
            for i in targetContigIDs:
                f.write("%s\n" % i )
    if toremove:
        with open(toremove, "w+") as f:
            for i in nontargetContigIDs:
                f.write("%s\n" % i)




#!/usr/bin/python
import click
import os

from sidr import default
from sidr import runfile

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


def validate_taxdump(value, method):
    if (os.path.isfile("%s/%s" % (value, "names.dmp")) and
        os.path.isfile("%s/%s" % (value, "nodes.dmp")) and
        os.path.isfile("%s/%s" % (value, "merged.dmp")) and
        os.path.isfile("%s/%s" % (value, "delnodes.dmp"))):
        return value
    else:
        with click.Context(method) as ctx:
            click.echo(ctx.get_help())
            raise click.BadParameter("Could not find names.dmp in taxdump, specify a value or make sure the files are present")


@click.group()
def cli():
    """
    Analyzes genomic data and attempts to classify contigs using a machine learning framework.

    SIDR uses data fron BLAST (or similar classifiers)to train a Decision Tree model to classify sequence data as either belonging to a target organism, or belonging to something else. This classification can be used to filter the data for later assembly.

    To use SIDR, you will need to construct a preliminary assembly, align your reads back to that assembly, then use BLAST to classify the assembly contigs.
    """
    pass


@cli.command(name="default", context_settings=CONTEXT_SETTINGS)
@click.option('--bam', '-b', type=click.Path(exists=True), help="Alignment of reads to preliminary assembly, in BAM format.")
@click.option('--fasta', '-f', type=click.Path(exists=True), help="Preliminary assembly, in FASTA format.")
@click.option('--blastresults', '-r', type=click.Path(exists=True), help="Classification of preliminary assembly from BLAST (or similar tools).")
@click.option('--taxdump', '-d', type=click.Path(), default=os.environ.get('BLASTDB'), help="Location of the NCBI Taxonomy dump. Default is $BLASTDB.")
@click.option('--model', '-m', 'modelOutput', type=click.Path(), default="", help="Location to save a graphical representation of the trained decision tree (optional). Output is in the form of a DOT file.")
@click.option('--output', '-o', type=click.Path(), default="%s/classifications.txt" % os.getcwd())
@click.option('--tokeep', '-k', type=click.Path(), default="", help="Location to save the contigs identified as the target organism(optional).")
@click.option('--toremove', '-x', type=click.Path(), default="", help="Location to save the contigs identified as not belonging to the target organism (optional).")
@click.option('--binary', is_flag=True, help="Use binary target/nontarget classification.")
@click.option('--target', '-t', help="The identity of the target organism at the chosen classification level. It is recommended to use the organism's phylum.")
@click.option('--level', '-l', default="phylum", help="The classification level to use when constructing the model. Default is 'phylum'.")
# @click.option('--verbose', '-v', count=True, help="Output more debugging options, repeat to increase verbosity (unimplemented).")
def default_runner(bam, fasta, blastresults, taxdump, modelOutput, output, tokeep, toremove, binary, target, level):
    """
    Runs the default analysis using raw preassembly data.
    """
    validate_taxdump(taxdump, runfile_runner)
    default.runAnalysis(bam, fasta, blastresults, taxdump, modelOutput, output, tokeep, toremove, binary, target, level)


@cli.command(name="runfile", context_settings=CONTEXT_SETTINGS)
@click.option('--infile', '-i', type=click.Path(exists=True), help="Tab-delimited input file.")
@click.option('--taxdump', '-d', type=click.Path(), default=os.environ.get('BLASTDB'), help="Location of the NCBI Taxonomy dump. Default is $BLASTDB.")
@click.option('--output', '-o', type=click.Path(), default="%s/classifications.txt" % os.getcwd())
@click.option('--model', '-m', 'modelOutput', type=click.Path(), default="", help="Location to save a graphical representation of the trained decision tree (optional). Output is in the form of a DOT file.")
@click.option('--tokeep', '-k', type=click.Path(), default="", help="Location to save the contigs identified as the target organism(optional).")
@click.option('--toremove', '-x', type=click.Path(), default="", help="Location to save the contigs identified as not belonging to the target organism (optional).")
@click.option('--target', '-t', help="The identity of the target organism at the chosen classification level. It is recommended to use the organism's phylum.")
@click.option('--binary', is_flag=True, help="Use binary target/nontarget classification.")
@click.option('--level', '-l', default="phylum", help="The classification level to use when constructing the model. Default is 'phylum'.")
def runfile_runner(infile, taxdump, output, modelOutput, tokeep, toremove, binary, target, level):
    """
    Runs a custom analysis using pre-computed data from BBMap or other sources.

    Input data will be read for all variables which will be used to construct a Decision Tree model.
    """
    validate_taxdump(taxdump, runfile_runner)
    runfile.runAnalysis(taxdump, infile, level, modelOutput, output, tokeep, toremove, binary, target)

""" WIP
@cli.command(name="filter", context_settings=CONTEXT_SETTINGS)
@click.option('--tokeep', '-k', type=click.Path(), default="", help="File containing list of contigs from the alignment to keep.")
@click.option('--bam', '-b', type=click.Path(exists=True), help="Alignment of reads to preliminary assembly, in BAM format.")
@click.option('-i1', type=click.Path(exists=True), help="Right read fastq to extract reads from.")
@click.option('-i2', type=click.Path(exists=True), help="Left read fastq to extract reads from.")
@click.option('-o1', type=click.Path(), help="Right read fastq to extract reads to.")
@click.option('-o2', type=click.Path(), help="Left read fastq to extract reads to.")
def filter_runner(tokeep, bam, i1, i2, o1, o2):
"""
#Filters reads aligning to the given contigs.
"""
    filterReads.runFilter(tokeep, bam, i1, i2, o1, o2)

if __name__ == "__main__":  # TODO Setuptools
    cli(prog_name="sidr")
""" 



import sidr
import pytest
import mock

from contextlib import closing
try:  # https://stackoverflow.com/questions/11914472/stringio-in-python3
    from StringIO import StringIO
except ImportError:
    from io import StringIO


# TODO: handle initial newline in csv
test_names = """1 |   root    |       |   scientific name |
2 |   phy    |       |   scientific name |
3 |   mergephy    |       |   scientific name |
"""
test_nodes = """1 |   1   |   no rank |       |   0   |   0   |   0   |   0   |   0   |   0   |   0   |   0   |
2 |   1   |   phylum |       |   0   |   0   |   0   |   0   |   0   |   0   |   0   |   0   |
3 |   1   |   phylum |       |   0   |   0   |   0   |   0   |   0   |   0   |   0   |   0   |
"""
test_deleted = "4 |"
test_merged = "5   |   3   |"
test_taxfiles = [test_names, test_nodes, test_merged, test_deleted]
test_taxdump = {'3': ['mergephy', '1', 'phylum'], '2': ['phy', '1', 'phylum'], '4': ['deleted'], '5': ['merged', '3'], '1': ['root', '1', 'no rank']}
test_gc = {"1": 50.0}

# some helpful sites
# https://stackoverflow.com/questions/1289894/how-do-i-mock-an-open-used-in-a-with-statement-using-the-mock-framework-in-pyth
# https://docs.python.org/dev/library/unittest.mock.html#mock-open
# http://boris-42.me/the-simplest-way-in-python-to-mock-open-during-unittest/
# https://gist.github.com/vpetro/1174019
# https://code.activestate.com/recipes/576650-with-statement-for-stringio/

def filegenerator(filestrings):
    for f in filestrings:
        yield closing(StringIO(f))


def test_parseTaxdump():
    with mock.patch('sidr.common.open', mock.mock_open()) as m:
        m.side_effect = filegenerator(test_taxfiles)
        assert sidr.common.parseTaxdump("test", False)[0] == test_taxdump


def test_taxidToLineage():
    assert sidr.common.taxidToLineage("2", test_taxdump, "phylum") == "phy"
    assert sidr.common.taxidToLineage("5", test_taxdump, "phylum") == "mergephy"
    with pytest.raises(Exception) as excinfo:
        sidr.common.taxidToLineage("4", test_taxdump, "phylum")
    assert "ERROR: Taxon id 4 has been deleted from the NCBI DB." in str(excinfo.value)

def test_constructCorpus():
    return


import pysam
import click
import gc
import gzip

from sidr import common
from Bio.SeqUtils import GC  # for GC content
from Bio.SeqIO.FastaIO import FastaIterator


def readFasta(fastaFile):
    """
    Reads a FASTA file and parses contigs for GC content.

    Args:
        fastaFile: The path to the FASTA file.
    Returns:
        contigs A dictionary mapping contigIDs to sidr.common.Contig objects with GC content as a variable.
    """
    contigs = []
    if ".gz" in fastaFile: # should support .fa.gz files in a seamless (if slow) way
        openFunc = gzip.open
    else:
        openFunc = open
    with openFunc(fastaFile) as data:
        click.echo("Reading %s" % fastaFile)
        with click.progressbar(FastaIterator(data)) as fi:
            for record in fi:  # TODO: conditional formatting
                contigs.append(common.Contig(record.id.split(' ')[0], variables={"GC": GC(record.seq)}))
    if len(contigs) != len(set([x.contigid for x in contigs])): # exit if duplicate contigs, https://stackoverflow.com/questions/5278122/checking-if-all-elements-in-a-list-are-unique
        raise ValueError("Input FASTA contains duplicate contigIDs, exiting")
    return dict((x.contigid, x) for x in contigs)  # https://stackoverflow.com/questions/3070242/reduce-python-list-of-objects-to-dict-object-id-object


def readBAM(BAMFile, contigs):
    """
    Parses an aligned BAM file for coverage.

    Args:
        BAMFile: The BAM file to parse.
        contigs: List of sidr.common.Contigs taken from input FASTA.
    Returns:
        contigs: Input contigs updated with coverage, measured as an
                 average over the whole contig.
    """
    alignment = pysam.AlignmentFile(BAMFile, "rb")
    click.echo("Reading BAM file")
    with click.progressbar(contigs) as ci:
        for contig in ci:
            covArray = [] # coverage over contig = sum(coverage per base)/number of bases
            for pile in alignment.pileup(region=str(contig)):
                covArray.append(pile.nsegments)
            try:
                contigs[contig].variables["Coverage"] = (sum(covArray) / len(covArray))
            except ZeroDivisionError: # should only occur if 0 coverage recorded
                contigs[contig].variables["Coverage"] = 0
    return contigs


def readBLAST(classification, taxdump, classificationLevel, contigs):
    """
    Reads a BLAST result file and combines it with other known information about the contigs.

    Args:
        classification: A string containing the filename of the BLAST results. The BLAST
            results must be in the format -outfmt '6 qseqid staxids', additional information
            can be added but the first two fields must be qseqid and staxids.
        taxdump: The NCBI taxdump as processed by parseTaxdump()
        classificationLevel: The level of classification to save into the corpus. Defaults to phylum.
        contigs: List of sidr.common.Contigs taken from input FASTA
    Returns:
        contigs: Input list of contigs updated with classification form BLAST
        classMap: A dictionary mapping class names to their class id used by scikit-learn.
        classList: A list of class names.
    """
    classList = []
    classMap = {}
    with open(classification) as data:
        click.echo("Reading %s" % classification)
        with click.progressbar(data) as dt:
            for line in dt:
                record = line.split("\t")
                contig = record[0]
                taxid = record[1].strip()
                taxonomy = common.taxidToLineage(taxid, taxdump, classificationLevel)
                taxonomy = taxonomy.lower()
                try:
                    if not contigs[contig].classification: # assume that the first hit in blast output is best
                        contigs[contig].classification = taxonomy
                        if taxonomy not in classList:
                            classList.append(taxonomy)
                except IndexError:  # if a contig is in BLAST but not FASTA (should be impossible but)
                    continue
    for idx, className in enumerate(classList):
        classMap[className] = idx
    return contigs, classMap, classList


def runAnalysis(bam, fasta, blastresults, taxdump, modelOutput, output, tokeep, toremove, binary, target, level):
    taxdump, taxidDict = common.parseTaxdump(taxdump, False)
    gc.collect()
    click.echo("Taxdump parsed, %d taxIDs loaded" % len(taxdump))
    contigs = readFasta(fasta)
    gc.collect()
    click.echo("FASTA loaded, %d contigs returned" % len(contigs))
    contigs = readBAM(bam, contigs)
    gc.collect()
    click.echo("BAM loaded")
    contigs, classMap, classList = readBLAST(blastresults,
                                             taxdump, level.lower(), contigs)
    gc.collect()
    click.echo("BLAST results loaded")
    corpus, testdata, features = common.constructCorpus(list(contigs.values()), classMap, binary, target)
    gc.collect()
    click.echo("Corpus constucted, %d contigs in corpus and %d contigs in test data" % (len(corpus), len(testdata)))
    classifier = common.constructModel(corpus, classList, features, modelOutput)
    result = common.classifyData(classifier, testdata, classMap)
    common.generateOutput(tokeep, toremove, result, contigs.values(), target, output)



import sidr
import pytest
import mock

from contextlib import closing
try:  # https://stackoverflow.com/questions/11914472/stringio-in-python3
    from StringIO import StringIO
except ImportError:
    from io import StringIO

test_fasta = """>1
GCGCATATGCGCATATGCGCATATGCGCATATGCGCATATGCGCATATGCGCATATGCGCATATGCGCATAT
GCGCATATGCGCATATGCGCATATGCGCATATGCGCATATGCGCATATGCGCATATGCGCATATGCGCATAT
GCGCATATGCGCATATGCGCATATGCGCATATGCGCATATGCGCATATGCGCATATGCGCATATGCGCATAT
"""

def filegenerator(filestrings):
    for f in filestrings:
        yield closing(StringIO(f))

def test_readFasta():
    with mock.patch('sidr.default.open', mock.mock_open()) as m:
        m.side_effect = filegenerator([test_fasta])
        fastaCheck = sidr.default.readFasta("test")
        assert list(fastaCheck.keys()) == ['1']
        assert list(fastaCheck.values())[0].variables["GC"] == 50

def test_readBAM():
    return


def test_readBLAST():
    return




import sidr
import pytest
import mock

from contextlib import closing
try:  # https://stackoverflow.com/questions/11914472/stringio-in-python3
    from StringIO import StringIO
except ImportError:
    from io import StringIO



import os
from setuptools import setup


setup(
    name="SIDR",
    version="0.0.1a0",
    author="Duncan Murdock",
    author_email="me@duncanmurdock.name",
    description=("Sequence Idenification using Decision tRees; a tool to classify DNA reads using machine learning models."),
    license="MIT",
    keywords="console tool bioinformatics",
    url="https://github.com/damurdock/SIDR",
    packages=['sidr', 'tests'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        'Environment :: Console',
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Science/Research",
    ],
    install_requires=["NumPy", "SciPy", "pysam", "BioPython", "scikit-learn", "click", "pandas"],
    long_description=open("README.rst").read(),
    entry_points={
        'console_scripts': [
            'sidr = sidr.cli:cli',
        ],
    })
